@@ -13,7 +13,7 @@ use indexmap::{IndexMap, IndexSet};
 const KNOWN_OPTIONS: [&str; 6] = ["-h", "-help", "--help", "-V", "-version", "--version"];
 
 pub(crate) struct Matcher<'a, 'b> {
-    cmds: Vec<(&'b str, &'a Command, String)>,
+    cmds: Vec<(&'b str, &'a Command, String, usize)>,
     args: &'b [String],
     flag_option_args: Vec<Vec<FlagOptionArg<'a, 'b>>>,
     positional_args: Vec<&'b str>,
@@ -36,6 +36,12 @@ pub(crate) enum ArgComp {
     Any,
 }
 
+impl ArgComp {
+    pub(crate) fn is_flag_or_option(&self) -> bool {
+        matches!(self, Self::FlagOrOption | &Self::OptionValue(_, _))
+    }
+}
+
 #[derive(Debug)]
 pub(crate) enum MatchError {
     DisplayHelp,
@@ -52,12 +58,13 @@ pub(crate) enum MatchError {
 
 impl<'a, 'b> Matcher<'a, 'b> {
     pub(crate) fn new(root: &'a Command, args: &'b [String]) -> Self {
-        let mut cmds = vec![(args[0].as_str(), root, args[0].clone())];
+        let mut cmds = vec![(args[0].as_str(), root, args[0].clone(), 0)];
         let mut cmd_level = 0;
         let mut arg_index = 1;
         let mut flag_option_args = vec![vec![]];
         let mut positional_args = vec![];
         let mut dashdash = vec![];
+        let mut terminated = false;
         let mut arg_comp = ArgComp::Any;
         let mut choices_fns = HashSet::new();
         let args_len = args.len();
@@ -73,10 +80,11 @@ impl<'a, 'b> Matcher<'a, 'b> {
             let arg = args[arg_index].as_str();
             if arg == "--" {
                 dashdash.push(positional_args.len());
-            } else if !dashdash.is_empty()
+            } else if terminated
+                || !dashdash.is_empty()
                 || (cmd.no_flags_options_subcommands() && !KNOWN_OPTIONS.contains(&arg))
             {
-                positional_args.push(arg);
+                add_positional_arg(&mut positional_args, arg, &mut terminated, cmd);
             } else if arg.starts_with('-') {
                 if let Some((k, v)) = arg.split_once('=') {
                     let param = cmd.find_flag_option(k);
@@ -130,13 +138,19 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     arg,
                     subcmd,
                     subcmd.name.clone().unwrap_or_else(|| arg.to_string()),
+                    arg_index,
                 ));
                 flag_option_args.push(vec![]);
             } else {
-                positional_args.push(arg);
+                add_positional_arg(&mut positional_args, arg, &mut terminated, cmd);
             }
             arg_index += 1;
         }
+
+        if terminated {
+            arg_comp = ArgComp::CommandOrPositional;
+        }
+
         let last_cmd = cmds.last().unwrap().1;
         choices_fns.extend(last_cmd.positional_params.iter().filter_map(|v| {
             if let Some((choices_fn, validate)) = v.choices_fn.as_ref() {
@@ -163,8 +177,19 @@ impl<'a, 'b> Matcher<'a, 'b> {
     pub(crate) fn set_script_path(&mut self, script_path: &str) {
         self.script_path = Some(script_path.to_string());
         let fns: Vec<&str> = self.choices_fns.iter().copied().collect();
-        if let Some(list) = run_param_fns(script_path, &fns, self.args, HashMap::new()) {
-            for (i, choices) in list.into_iter().enumerate() {
+        if let Some(outputs) = run_param_fns(script_path, &fns, self.args, HashMap::new()) {
+            for (i, output) in outputs.into_iter().enumerate() {
+                let choices = output
+                    .split('\n')
+                    .filter_map(|v| {
+                        let v = v.trim_matches(|c: char| c.is_whitespace() || c == '\0');
+                        if v.is_empty() {
+                            None
+                        } else {
+                            Some(v.to_string())
+                        }
+                    })
+                    .collect();
                 self.choices_values.insert(fns[i], choices);
             }
         }
@@ -199,15 +224,13 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 self.dashdash.iter().map(|v| v.to_string()).collect(),
             ));
         }
-        output.push(ArgcValue::Multiple(
-            "_args".into(),
-            self.positional_args.iter().map(|v| v.to_string()).collect(),
-        ));
         output
     }
 
     pub(crate) fn compgen(&self) -> Vec<(String, String)> {
-        match &self.arg_comp {
+        let level = self.cmds.len() - 1;
+        let mut last_cmd = self.cmds[level].1;
+        let mut output = match &self.arg_comp {
             ArgComp::FlagOrOption => self.comp_flag_options(),
             ArgComp::FlagOrOptionCombine(value) => {
                 let mut output: Vec<(String, String)> = self
@@ -227,49 +250,46 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 output
             }
             ArgComp::CommandOrPositional => {
-                let level = self.cmds.len() - 1;
-                let mut cmd = self.cmds[level].1;
                 if self.positional_args.len() == 2 && self.positional_args[0] == "help" {
-                    return comp_subcomands(cmd);
+                    return comp_subcomands(last_cmd);
                 }
                 if level > 0
                     && self.positional_args.is_empty()
                     && self.flag_option_args[level].is_empty()
                 {
-                    cmd = self.cmds[level - 1].1;
+                    last_cmd = self.cmds[level - 1].1;
                 }
                 let values = self.match_positionals();
-                comp_subcommands_positional(cmd, &values, self.positional_args.len() < 2)
+                comp_subcommands_positional(last_cmd, &values, self.positional_args.len() < 2)
             }
             ArgComp::OptionValue(name, index) => {
-                let cmd = self.cmds[self.cmds.len() - 1].1;
-                if let Some(param) = cmd.flag_option_params.iter().find(|v| &v.name == name) {
+                if let Some(param) = last_cmd.flag_option_params.iter().find(|v| &v.name == name) {
                     comp_flag_option(param, *index)
                 } else {
                     vec![]
                 }
             }
             ArgComp::Any => {
-                let cmd = self.cmds[self.cmds.len() - 1].1;
                 if self.positional_args.len() == 2 && self.positional_args[0] == "help" {
-                    return comp_subcomands(cmd);
+                    return comp_subcomands(last_cmd);
                 }
                 let mut output = vec![];
-                if cmd.positional_params.is_empty() {
+                if last_cmd.positional_params.is_empty() {
                     output.extend(self.comp_flag_options());
                 }
                 let values = self.match_positionals();
                 output.extend(comp_subcommands_positional(
-                    cmd,
+                    last_cmd,
                     &values,
                     self.positional_args.len() < 2,
                 ));
-                if output.is_empty() {
-                    output.push(("__argc_value:file".into(), String::new()))
-                }
                 output
             }
+        };
+        if output.is_empty() && !self.arg_comp.is_flag_or_option() {
+            output.push(("__argc_value:file".into(), String::new()));
         }
+        output
     }
 
     fn to_arg_values_base(&self) -> Vec<ArgcValue> {
@@ -277,6 +297,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let cmds_len = self.cmds.len();
         let level = cmds_len - 1;
         let last_cmd = self.cmds[level].1;
+        let args_index = self.cmds[level].3;
         for level in 0..cmds_len {
             let args = &self.flag_option_args[level];
             let cmd = self.cmds[level].1;
@@ -307,6 +328,8 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 output.push(value);
             }
         }
+        output.push(ArgcValue::Multiple("_args".into(), self.args.to_vec()));
+        output.push(ArgcValue::Single("_index".into(), args_index.to_string()));
         output
     }
 
@@ -354,11 +377,13 @@ impl<'a, 'b> Matcher<'a, 'b> {
             }
         }
         if !last_cmd.subcommands.is_empty() {
-            if self.positional_args.is_empty() && last_args.is_empty() {
-                if !last_cmd.exist_main_fn(&cmd_paths) {
+            if !last_cmd.exist_main_fn(&cmd_paths) {
+                if self.positional_args.is_empty() && last_args.is_empty() {
                     return Some(MatchError::DisplayHelp);
+                } else {
+                    return Some(MatchError::InvalidSubcommand);
                 }
-            } else {
+            } else if last_cmd.positional_params.is_empty() && !self.positional_args.is_empty() {
                 return Some(MatchError::InvalidSubcommand);
             }
         }
@@ -671,6 +696,21 @@ impl<'a, 'b> Matcher<'a, 'b> {
     }
 }
 
+fn add_positional_arg<'a>(
+    positional_args: &mut Vec<&'a str>,
+    arg: &'a str,
+    terminated: &mut bool,
+    cmd: &Command,
+) {
+    positional_args.push(arg);
+    if !*terminated
+        && cmd.positional_params.last().map(|v| v.terminated) == Some(true)
+        && positional_args.len() >= cmd.positional_params.len() - 1
+    {
+        *terminated = true;
+    }
+}
+
 fn take_value_args(args: &[String], start: usize, len: usize) -> Vec<&str> {
     let mut output = vec![];
     if len == 0 {
@@ -713,24 +753,37 @@ fn match_flag_option<'a, 'b>(
     param: &'a FlagOptionParam,
     arg_comp: &mut ArgComp,
 ) {
-    let values_len = param.values_size();
-    let args_len = args.len();
-    let value_args = take_value_args(args, *arg_index + 1, values_len);
-    let arg = &args[*arg_index];
-    *arg_index += value_args.len();
-    if *arg_index == args_len - 1 {
-        if *arg_comp != ArgComp::FlagOrOption && param.is_option() && value_args.len() <= values_len
-        {
+    if param.terminated {
+        let value_args: Vec<&str> = args[*arg_index + 1..].iter().map(|v| v.as_str()).collect();
+        let arg = &args[*arg_index];
+        *arg_index += value_args.len();
+        if !value_args.is_empty() {
             *arg_comp =
                 ArgComp::OptionValue(param.name.clone(), value_args.len().saturating_sub(1));
-        } else if *arg_comp == ArgComp::FlagOrOption
-            && param.is_flag()
-            && !(arg.len() > 2 && param.is_match(arg))
-        {
-            *arg_comp = ArgComp::FlagOrOptionCombine(arg.to_string());
         }
+        output.push((arg, value_args, Some(param.name.as_str())));
+    } else {
+        let values_len = param.values_size();
+        let args_len = args.len();
+        let value_args = take_value_args(args, *arg_index + 1, values_len);
+        let arg = &args[*arg_index];
+        *arg_index += value_args.len();
+        if *arg_index == args_len - 1 {
+            if *arg_comp != ArgComp::FlagOrOption
+                && param.is_option()
+                && value_args.len() <= values_len
+            {
+                *arg_comp =
+                    ArgComp::OptionValue(param.name.clone(), value_args.len().saturating_sub(1));
+            } else if *arg_comp == ArgComp::FlagOrOption
+                && param.is_flag()
+                && !(arg.len() > 2 && param.is_match(arg))
+            {
+                *arg_comp = ArgComp::FlagOrOptionCombine(arg.to_string());
+            }
+        }
+        output.push((arg, value_args, Some(param.name.as_str())));
     }
-    output.push((arg, value_args, Some(param.name.as_str())));
 }
 
 fn comp_subcommands_positional(
@@ -771,6 +824,7 @@ fn comp_flag_option(param: &FlagOptionParam, index: usize) -> Vec<(String, Strin
         value_name,
         &param.choices,
         &param.choices_fn,
+        &param.multi_char,
     )
 }
 
@@ -780,6 +834,7 @@ fn comp_positional(param: &PositionalParam) -> Vec<(String, String)> {
         &param.arg_value_name,
         &param.choices,
         &param.choices_fn,
+        &param.multi_char,
     )
 }
 
@@ -788,6 +843,7 @@ fn comp_param(
     value_name: &str,
     choices: &Option<Vec<String>>,
     choices_fn: &Option<(String, bool)>,
+    multi_char: &Option<char>,
 ) -> Vec<(String, String)> {
     let choices: Option<Either<Vec<String>, String>> = if let Some(choices_fn) = choices_fn {
         Some(Either::Right(choices_fn.0.to_string()))
@@ -796,7 +852,7 @@ fn comp_param(
             .as_ref()
             .map(|choices| Either::Left(choices.iter().map(|v| v.to_string()).collect()))
     };
-    if let Some(choices) = choices {
+    let mut output = if let Some(choices) = choices {
         match choices {
             Either::Left(choices) => choices
                 .iter()
@@ -807,7 +863,11 @@ fn comp_param(
     } else {
         let value = format!("__argc_value:{}", value_name);
         vec![(value, describe.into())]
+    };
+    if let Some(ch) = multi_char {
+        output.insert(0, (format!("__argc_multi:{}", ch), String::new()));
     }
+    output
 }
 
 fn get_param_choices<'a, 'b: 'a>(
