@@ -166,6 +166,7 @@ fn choice() {
     let script = r###"
 # @option --oa[`_choice_fn`]
 # @option --ob[x|y|z]
+# @option --oc*,[`_choice_fn`]
 # @arg v1[x|y|z]
 # @arg v2[`_choice_fn`]
 _choice_fn() {
@@ -187,6 +188,8 @@ _choice_fn() {
             vec!["prog", "'--oa=a"],
             vec!["prog", "\"--oa="],
             vec!["prog", "\"--oa=a"],
+            vec!["prog", "--oc", ""],
+            vec!["prog", "--oc", "abc,"],
         ]
     );
 }
@@ -349,30 +352,6 @@ arg() { :; }
     );
 }
 
-#[test]
-fn parts() {
-    let script = r###"
-# @option --oa*[`_choice_fn`]
-_choice_fn() {
-    echo __argc_parts:/
-	echo A/B
-	echo A/B/C
-    echo A/C
-    echo B/C
-    echo C
-}
-"###;
-
-    snapshot_compgen!(
-        script,
-        vec![
-            vec!["prog", "--oa", ""],
-            vec!["prog", "--oa", "A/"],
-            vec!["prog", "--oa", "A/B/"],
-        ]
-    );
-}
-
 #[test]
 fn no_space() {
     let script = r###"
@@ -403,15 +382,14 @@ _choice_fn() {
 }
 
 #[test]
-fn parts_shell() {
+fn multi_parts() {
     let script = r###"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
-    echo __argc_parts:/
-	echo A/B
-	echo A/B/
-	echo A/B/C
-	echo A/B/D
+    echo __argc_prefix:A/
+    echo __argc_matcher:''
+	echo B
+	echo -e "B/\0"
 }
 "###;
 
@@ -419,17 +397,225 @@ _choice_fn() {
 }
 
 #[test]
-fn parts2_shell() {
+fn multi_parts2() {
     let script = r###"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
-    echo __argc_parts:/
-	echo A/B
-	echo A/B/
-	echo A/B/C
-	echo A/B/D
+    echo __argc_prefix:A/B/
+    echo __argc_matcher:''
+	echo
+	echo C
+	echo D
 }
 "###;
 
     snapshot_compgen_shells!(script, vec!["prog", "--oa", "A/B/"]);
 }
+
+#[test]
+fn suffix() {
+    let script = r###"
+# @option --oa*[`_choice_fn`]
+_choice_fn() {
+    echo __argc_suffix:=
+    echo -e "A\0"
+    echo -e "B\0"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
+}
+
+#[test]
+fn option_equal() {
+    let script = r###"
+# @option --oa[`_choice_fn`]
+# @arg val[`_choice_fn`]
+_choice_fn() {
+    echo __argc_matcher:
+    ( set -o posix ; set ) | grep 'argc_\|ARGC_MATCHER'
+}
+"###;
+
+    snapshot_compgen!(
+        script,
+        vec![vec!["prog", "--oa=abc"], vec!["prog", "oa=abc"],]
+    );
+}
+
+#[test]
+fn arg_terminated() {
+    let script = r###"
+# @arg cmd
+# @arg args~[`_choice_fn`]
+_choice_fn() {
+    echo __argc_matcher:
+    echo ok
+}
+"###;
+
+    snapshot_compgen!(
+        script,
+        vec![
+            vec!["sudo", "cmd", ""],
+            vec!["sudo", "cmd", "-"],
+            vec!["sudo", "cmd", "--foo"],
+            vec!["sudo", "cmd", "foo"],
+        ]
+    );
+}
+
+#[test]
+fn option_terminated() {
+    let script = r###"
+# @option --oa~[`_choice_fn`]
+# @option --ob
+_choice_fn() {
+    echo __argc_matcher:
+    echo ok
+}
+"###;
+
+    snapshot_compgen!(
+        script,
+        vec![
+            vec!["prog", "--oa"],
+            vec!["prog", "--oa", ""],
+            vec!["prog", "--oa", "--"],
+            vec!["prog", "--oa", "v1", "v2"],
+            vec!["prog", "--oa", "--", ""],
+        ]
+    );
+}
+
+#[test]
+fn fallback_comp_file() {
+    let script = r###"
+# @cmd
+args() {
+    :;
+}
+
+# @cmd
+# @option --file
+# @option --value
+cmd() {
+    :;
+}
+"###;
+
+    snapshot_compgen!(
+        script,
+        vec![
+            vec!["prog", "args", ""],
+            vec!["prog", "args", "v"],
+            vec!["prog", "cmd", "--file", ""],
+            vec!["prog", "cmd", "--file", "v"],
+            vec!["prog", "cmd", "--value", ""],
+            vec!["prog", "cmd", "--value", "v"],
+        ]
+    );
+}
+
+#[test]
+fn mult_char() {
+    let script = r###"
+# @option --oa*,[`_choice_fn`]
+_choice_fn() {
+    echo -e "abc\ndef\nijk"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "--oa", "abc,"]);
+}
+
+#[test]
+fn mult_char2() {
+    let script = r###"
+# @option --oa*,[`_choice_fn`]
+_choice_fn() {
+    echo -e "abc\ndef\nijk"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "--oa=abc,"]);
+}
+
+#[test]
+fn starts_quote() {
+    let script = r###"
+# @option --oa[`_choice_fn`]
+_choice_fn() {
+    echo -e "abc\ndef\nijk"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "--oa='"]);
+}
+
+#[test]
+fn starts_quote2() {
+    let script = r###"
+# @option --oa[`_choice_fn`]
+_choice_fn() {
+    echo -e "abc\ndef\nijk"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "'--oa="]);
+}
+
+#[test]
+fn desc() {
+    let script = r###"
+# @option --oa[`_choice_fn`]
+_choice_fn() {
+    echo -e "abc\t(desc1)"
+    echo -e "def\t(desc2)"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
+}
+
+#[test]
+fn desc2() {
+    let script = r###"
+# @option --oa[`_choice_fn`]
+_choice_fn() {
+    echo -e "abc\t(desc1)"
+    echo -e "def\t(desc2)"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "--oa", "a"]);
+}
+
+#[test]
+fn escape() {
+    let script = r###"
+# @option --oa[`_choice_fn`]
+_choice_fn() {
+    echo -e "a:b>c"
+    echo -e "d:e>f"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
+}
+
+#[test]
+fn generic_shell() {
+    let script = r###"
+# @option --oa[`_choice_fn`]
+_choice_fn() {
+    echo -e "__argc_prefix:'"
+    echo -e "__argc_matcher:"
+    echo -e "abc\t(desc 1)"
+    echo -e "def\0"
+    echo -e "ijk"
+}
+"###;
+
+    snapshot_compgen!(script, vec![vec!["prog", "--oa", ""]], argc::Shell::Generic);
+}
